﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Drawing;
using System.IO;

namespace Browser
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            treeView.ShowNodeToolTips = true;
            listView.ShowItemToolTips = true;
            tabControl.ShowToolTips = true;

            // DirectoryInfo dir = new DirectoryInfo(".");
            // var dir = new DirectoryInfo("C:\\");

            DriveInfo [] roots = DriveInfo.GetDrives ();
            foreach (DriveInfo r in roots)
               displayBranch (treeView.Nodes, r.RootDirectory);
        }
        void displayBranch (TreeNodeCollection target, DirectoryInfo dir, int levels = 2)
        {
            MyNode node = new MyNode ();
            node.path = dir.FullName;
            node.ready = false;
            node.Text = dir.Name;
            node.ToolTipText = node.path;
            node.ForeColor = Color.Blue;
            target.Add (node);
            displayDetail (node, dir, levels);
        }
        void displayDetail (MyNode node, DirectoryInfo dir, int levels)
        { 
            if (levels > 1)
                try  {
                    foreach (DirectoryInfo d in dir.GetDirectories())
                        displayBranch (node.Nodes, d, levels - 1);
                    node.ready = true;
                }
                catch (Exception e) {
                    TreeNode t = new TreeNode ();
                    t.Text = e.ToString ();
                    t.ToolTipText = e.StackTrace;
                    t.ForeColor = Color.Red;
                    node.Nodes.Add (t);
                }
        }
        private void treeView_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            foreach (TreeNode n in e.Node.Nodes)
            {
                MyNode node = n as MyNode;
                if (node != null && ! node.ready)
                {
                    try
                    {
                        DirectoryInfo dir = new DirectoryInfo(node.path);
                        displayDetail (node, dir, 2);
                    }
                    catch (Exception ex)
                    {
                    }
                }
            }
        }

        private void treeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            MyNode node = e.Node as MyNode;
            if (node != null)
            {
                listView.Items.Clear ();
                try
                {
                    var dir = new DirectoryInfo(node.path);
                    FileInfo [] files = dir.GetFiles();
                    foreach (var f in files)
                    {
                        ListViewItem item = new ListViewItem();
                        item.Text = f.Name;
                        item.ToolTipText = f.FullName;
                        item.Tag = f.FullName;
                        long n = f.Length;
                        String s = String.Format("{0:### ### ### ##0}", n);
                        item.SubItems.Add (s);
                        item.SubItems.Add(f.LastWriteTime.ToString ("yyyy-MM-dd HH:mm:ss"));
                        listView.Items.Add(item);
                    }
                }
                catch (Exception ex)
                {
                }
            }
        } // end of treeView_NodeMouseClick

        private void listView_ItemActivate(object sender, EventArgs e)
        {
            foreach (ListViewItem item in listView.SelectedItems)
            {
                String path = item.Tag as String;
                statusLabel.Text = path;
                showFile(path);
            }
        }

        private void showFile (string path)
        {
            FileInfo f = new FileInfo (path);
            string ext = f.Extension;
            switch (ext)
            {
                case ".txt":
                case ".h":
                case ".cpp":
                case ".cs":
                case ".ini":
                    showTextFile(f);
                    break;
            }
        }

        private void showTextFile (FileInfo f)
        {
            StreamReader r = f.OpenText ();
            string s = r.ReadToEnd ();
            // textBox.Text = s;
            // tabControl.SelectedTab = textPage;

            var page = new TabPage ();
            page.Text = f.Name;
            page.ToolTipText = f.FullName;
            tabControl.TabPages.Add(page);
            tabControl.SelectedTab = page;

            var box = new TextBox();
            box.Multiline = true;
            box.Dock = DockStyle.Fill;
            box.Text = s;
            page.Controls.Add (box);
        }
    }
}
